//
// ETSI (European Telecommunications Standards Institute, referred to as ETSI)
// ARF - ETSI ISG Augmented Reality Framework (ISG ARF)
// (C) 2022
//
// Development "World Storage", data management, authoring tools
//
// Authors:
// - Sylvain Renault (Fraunhofer HHI)
// 
// Date: Feb. 2022
//

#define USING_OPENAPI_GENERATOR // alt. is Swagger

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using ETSI.ARF.WorldStorage.RESTful;

#if USING_OPENAPI_GENERATOR
using Org.OpenAPITools.Api;
using Org.OpenAPITools.Model;
#else
using IO.Swagger.Api;
using IO.Swagger.Model;
#endif


namespace ETSI.ARF.WorldStorage.UI
{
    public class WorldStorageWindow : EditorWindow
    {
        static public WorldStorageWindow WorldStorageWindowSingleton;

        [HideInInspector] public WorldStorageSettings worldStorageSettings;

        [SerializeField] public List<string> creators = new List<string>();
        [SerializeField] public List<string> trackables = new List<string>();

        string vers = "Unknow";

        private Vector2 scrollPos;
        private Color ori;
        private GUIStyle gsTest;

        static public string winName = "ARF Authoring Editor";
        static public Color[] arfColors = new Color[] 
        {
            Color.yellow,                   // paneltext
            new Color(0.3f, 1f, 1f),        // button REST
            new Color(1f, 1f, 0.3f),        // button create + window
            new Color(1f, 0f, 0f),          // button delete (red)
            new Color(1f, 0f, 1f),           // button graph
            new Color(.7f, .5f, 1f)           // button prefab
        }; 

        
        //[MenuItem("[ ISG-ARF ]/World Storage Editor")]
        public static void ShowWindow()
        {
            EditorWindow.GetWindow(typeof(WorldStorageWindow), false, WorldStorageWindow.winName);
        }

        public WorldStorageWindow()
        {
            WorldStorageWindowSingleton = this;
        }
        static public void DrawCopyright()
        {
            // Title 
            GUILayout.Label("Augmented Reality Framework", EditorStyles.boldLabel);
            GUILayout.Label("Copyright(c) 2022, ETSI (BSD 3-Clause License)");
        }

        void OnGUI()
        {
            ori = GUI.backgroundColor;
            gsTest = new GUIStyle("window");
            gsTest.fontStyle = FontStyle.Bold;
            gsTest.normal.textColor = WorldStorageWindow.arfColors[0];

            scrollPos = EditorGUILayout.BeginScrollView(scrollPos, GUILayout.ExpandWidth(true));
            WorldStorageWindow.DrawCopyright();
            DrawServerStuffs();

            GUI.backgroundColor = WorldStorageWindow.arfColors[2];
            if (GUILayout.Button("Create/Edit Trackable..."))
            {
                Debug.Log("Open Trackable Window");
                TrackableWindow.ShowWindow(worldStorageSettings);
            }
            if (GUILayout.Button("Create/Edit World Anchor..."))
            {
                Debug.Log("Open World Anchor Window");
                WorldAnchorWindow.ShowWindow(worldStorageSettings);
            }
            if (GUILayout.Button("Create/Edit Link..."))
            {
            }

            GUI.backgroundColor = WorldStorageWindow.arfColors[4];
            if (GUILayout.Button("Show Graph..."))
            {
                GraphWindow.ShowWindow();
            }
            EditorGUILayout.EndScrollView();
        }

        public void UpdateList()
        {
            // Get all objects
            Debug.Log("Get all server objects");
            List<Trackable> res = RESTfulTrackableRequest.GetAllTrackables(worldStorageSettings);
            trackables.Clear();
            foreach (var item in res)
            {
                trackables.Add(item.UID.ToString());
            }
        }

        void DrawServerStuffs()
        {
            // Server info
            GUILayout.BeginVertical("World Storage Server", gsTest);
            //
            GUILayout gl = new GUILayout();
            
            GUILayout.Label("Server Name: " + worldStorageSettings.serverName, EditorStyles.whiteLargeLabel);
            GUILayout.Label("Own Creator UID: " + worldStorageSettings.creatorUID);
            GUILayout.Label("Base Path: " + worldStorageSettings.basePath);
            GUILayout.Label("Port: " + worldStorageSettings.port);

            EditorGUILayout.Space();

            // Ping
            EditorGUILayout.Space();
            if (GUILayout.Button("Ping"))
            {
                vers = RESTfulAdminRequest.Ping(worldStorageSettings);
            }

            // Version
            GUILayout.BeginHorizontal();
            vers = EditorGUILayout.TextField("Version", vers);

            if (GUILayout.Button("Get World Storage API Version"))
            {
                vers = RESTfulAdminRequest.GetAdminInfo(worldStorageSettings);
            }
            GUI.backgroundColor = ori;
            GUILayout.EndHorizontal();

            ScriptableObject target = this;
            SerializedObject so = new SerializedObject(target);

            // Get creators
            #region Get all creatorUid
            EditorGUILayout.Space(10);
            GUI.backgroundColor = WorldStorageWindow.arfColors[1];
            if (GUILayout.Button("Request all Creator ID"))
            {
                // Get all objects
                Debug.Log("Get all creators id");
                List<Trackable> res = RESTfulTrackableRequest.GetAllTrackables(worldStorageSettings);
                creators.Clear();
                foreach (var item in res)
                {
                    if (!creators.Contains(item.CreatorUID.ToString())) creators.Add(item.CreatorUID.ToString());
                }
            }
            GUI.backgroundColor = ori;

            SerializedProperty stringsProperty = so.FindProperty("creators");
            EditorGUILayout.PropertyField(stringsProperty, true); // True means show children
            so.ApplyModifiedProperties(); // Remember to apply modified properties
            #endregion

            // Get trackables
            #region Get all trackable objects
            EditorGUILayout.Space(10);
            GUI.backgroundColor = WorldStorageWindow.arfColors[1];
            if (GUILayout.Button("Request all Trackable ID"))
            {
                UpdateList();
            }
            GUI.backgroundColor = ori;
            //GUILayout.Label("Number of Trackables: " + trackables.Count.ToString());

            GUI.backgroundColor = WorldStorageWindow.arfColors[3];
            if (GUILayout.Button("Delete all Trackables (5 stay in!!!)"))
            {
                Debug.Log("Deleting all Trackable ");
                int n = 0;
                foreach (var item in trackables)
                {
                    if (++n > 5) RESTfulTrackableRequest.DeleteTrackable(worldStorageSettings, item);
                }
                
                UpdateList();
                WorldStorageWindow.WorldStorageWindowSingleton.Repaint();
            }
            GUI.backgroundColor = ori;

            stringsProperty = so.FindProperty("trackables");
            EditorGUILayout.PropertyField(stringsProperty, /*new GUIContent("Trackbales"),*/ true); // True means show children
            so.ApplyModifiedProperties(); // Remember to apply modified properties
            #endregion

            // Anchors
            #region Get all anchor objects
            GUI.backgroundColor = WorldStorageWindow.arfColors[1];
            if (GUILayout.Button("Request all Anchor ID"))
            {
                
            }
            GUI.backgroundColor = ori;
            #endregion

            // Links
            #region Get all link objects
            GUI.backgroundColor = WorldStorageWindow.arfColors[1];
            if (GUILayout.Button("Request all Link ID"))
            {

            }
            GUI.backgroundColor = ori;
            #endregion

            //
            GUILayout.EndVertical();
        }
    }
}